@@ -3,6 +3,7 @@
 //! <https://en.wikipedia.org/wiki/Differential_evolution>
 use self::Strategy::*;
 use crate::utility::prelude::*;
+use alloc::{vec, vec::Vec};
 
 /// The Differential Evolution strategy.
 ///
@@ -50,25 +51,25 @@ pub enum Strategy {
 
 /// Differential Evolution settings.
 pub struct De {
-    base: BasicSetting,
     strategy: Strategy,
     f: f64,
     cross: f64,
 }
 
 impl De {
-    impl_builder!(strategy, Strategy, "Strategy of the formula.");
-    impl_builder!(f, f64, "F factor.");
-    impl_builder!(cross, f64, "Crossing probability.");
+    impl_builders! {
+        /// Strategy of the formula.
+        fn strategy(Strategy)
+        /// F factor.
+        fn f(f64)
+        /// Crossing probability.
+        fn cross(f64)
+    }
 }
 
 impl Default for De {
     fn default() -> Self {
         Self {
-            base: BasicSetting {
-                pop_num: 400,
-                ..Default::default()
-            },
             strategy: S1,
             f: 0.6,
             cross: 0.9,
@@ -79,15 +80,7 @@ impl Default for De {
 impl Setting for De {
     type Algorithm = Method;
 
-    fn base(&self) -> &BasicSetting {
-        &self.base
-    }
-
-    fn base_mut(&mut self) -> &mut BasicSetting {
-        &mut self.base
-    }
-
-    fn create(self) -> Self::Algorithm {
+    fn algorithm(self) -> Self::Algorithm {
         let num = match self.strategy {
             S1 | S3 | S6 | S8 => 2,
             S2 | S7 => 3,
@@ -101,6 +94,13 @@ impl Setting for De {
             strategy: self.strategy,
         }
     }
+
+    fn default_basic() -> BasicSetting {
+        BasicSetting {
+            pop_num: 400,
+            ..Default::default()
+        }
+    }
 }
 
 /// Differential Evolution type.
@@ -112,7 +112,7 @@ pub struct Method {
 }
 
 impl Method {
-    fn f45<F: ObjFunc>(&self, ctx: &Context<F>, v: &Array1<usize>, n: usize) -> f64 {
+    fn f45<F: ObjFunc>(&self, ctx: &Context<F>, v: &[usize], n: usize) -> f64 {
         (ctx.pool[[v[0], n]] + ctx.pool[[v[1], n]] - ctx.pool[[v[2], n]] - ctx.pool[[v[3], n]])
             * self.f
     }
@@ -121,7 +121,7 @@ impl Method {
         &self,
         ctx: &Context<F>,
         tmp: &Array1<f64>,
-        v: &Array1<usize>,
+        v: &[usize],
         n: usize,
     ) -> f64 {
         match self.strategy {
@@ -139,13 +139,13 @@ impl Method {
         &mut self,
         ctx: &Context<F>,
         tmp: &mut Array1<f64>,
-        v: Array1<usize>,
+        v: Vec<usize>,
         mut n: usize,
     ) {
         for _ in 0..ctx.dim() {
             tmp[n] = self.formula(ctx, tmp, &v, n);
             n = (n + 1) % ctx.dim();
-            if !maybe(self.cross) {
+            if !ctx.rng.maybe(self.cross) {
                 break;
             }
         }
@@ -155,11 +155,11 @@ impl Method {
         &mut self,
         ctx: &Context<F>,
         tmp: &mut Array1<f64>,
-        v: Array1<usize>,
+        v: Vec<usize>,
         mut n: usize,
     ) {
         for lv in 0..ctx.dim() {
-            if !maybe(self.cross) || lv == ctx.dim() - 1 {
+            if !ctx.rng.maybe(self.cross) || lv == ctx.dim() - 1 {
                 tmp[n] = self.formula(ctx, tmp, &v, n);
             }
             n = (n + 1) % ctx.dim();
@@ -171,11 +171,11 @@ impl<F: ObjFunc> Algorithm<F> for Method {
     fn generation(&mut self, ctx: &mut Context<F>) {
         'a: for i in 0..ctx.pop_num() {
             // Generate Vector
-            let mut v = Array1::zeros(self.num);
-            rand_vector(v.as_slice_mut().unwrap(), 0, 0, ctx.pop_num());
+            let mut v = vec![0; self.num];
+            ctx.rng.vector(&mut v, 0, 0..ctx.pop_num());
             // Recombination
             let mut tmp = ctx.pool.slice(s![i, ..]).to_owned();
-            let n = rand_int(0, ctx.dim());
+            let n = ctx.rng.int(0..ctx.dim());
             match self.strategy {
                 S1 | S2 | S3 | S4 | S5 => self.c1(ctx, &mut tmp, v, n),
                 S6 | S7 | S8 | S9 | S10 => self.c2(ctx, &mut tmp, v, n),
