@@ -5,25 +5,25 @@ use crate::utility::prelude::*;
 
 /// Particle Swarm Optimization settings.
 pub struct Pso {
-    base: BasicSetting,
     cognition: f64,
     social: f64,
     velocity: f64,
 }
 
 impl Pso {
-    impl_builder!(cognition, f64, "Cognition factor.");
-    impl_builder!(social, f64, "Social factor.");
-    impl_builder!(velocity, f64, "Moving velocity.");
+    impl_builders! {
+        /// Cognition factor.
+        fn cognition(f64)
+        /// Social factor.
+        fn social(f64)
+        /// Moving velocity.
+        fn velocity(f64)
+    }
 }
 
 impl Default for Pso {
     fn default() -> Self {
         Self {
-            base: BasicSetting {
-                pop_num: 200,
-                ..Default::default()
-            },
             cognition: 2.05,
             social: 2.05,
             velocity: 1.3,
@@ -34,15 +34,7 @@ impl Default for Pso {
 impl Setting for Pso {
     type Algorithm = Method;
 
-    fn base(&self) -> &BasicSetting {
-        &self.base
-    }
-
-    fn base_mut(&mut self) -> &mut BasicSetting {
-        &mut self.base
-    }
-
-    fn create(self) -> Self::Algorithm {
+    fn algorithm(self) -> Self::Algorithm {
         Method {
             cognition: self.cognition,
             social: self.social,
@@ -51,6 +43,13 @@ impl Setting for Pso {
             best_past_f: Array1::ones(1) * f64::INFINITY,
         }
     }
+
+    fn default_basic() -> BasicSetting {
+        BasicSetting {
+            pop_num: 200,
+            ..Default::default()
+        }
+    }
 }
 
 /// Particle Swarm Optimization type.
@@ -81,8 +80,8 @@ impl<F: ObjFunc> Algorithm<F> for Method {
         #[cfg(not(feature = "parallel"))]
         {
             zip.for_each(|f, mut v, mut past, f_past| {
-                let alpha = rand_float(0., self.cognition);
-                let beta = rand_float(0., self.social);
+                let alpha = ctx.rng.float(0.0..self.cognition);
+                let beta = ctx.rng.float(0.0..self.social);
                 for s in 0..ctx.dim() {
                     let variable = self.velocity * v[s]
                         + alpha * (past[s] - v[s])
@@ -102,8 +101,8 @@ impl<F: ObjFunc> Algorithm<F> for Method {
             let (f, v) = zip
                 .into_par_iter()
                 .map(|(f, mut v, mut past, past_f)| {
-                    let alpha = rand_float(0., self.cognition);
-                    let beta = rand_float(0., self.social);
+                    let alpha = ctx.rng.float(0.0..self.cognition);
+                    let beta = ctx.rng.float(0.0..self.social);
                     for s in 0..ctx.dim() {
                         let variable = self.velocity * v[s]
                             + alpha * (past[s] - v[s])
