@@ -11,7 +11,6 @@ use core::marker::PhantomData;
 
 /// Real-coded Genetic Algorithm settings.
 pub struct Rga<R: Respond> {
-    base: BasicSetting,
     cross: f64,
     mutate: f64,
     win: f64,
@@ -20,19 +19,21 @@ pub struct Rga<R: Respond> {
 }
 
 impl<R: Respond> Rga<R> {
-    impl_builder!(cross, f64, "Crossing probability.");
-    impl_builder!(mutate, f64, "Mutation probability.");
-    impl_builder!(win, f64, "Winning probability.");
-    impl_builder!(delta, f64, "Delta factor.");
+    impl_builders! {
+        /// Crossing probability.
+        fn cross(f64)
+        /// Mutation probability.
+        fn mutate(f64)
+        /// Winning probability.
+        fn win(f64)
+        /// Delta factor.
+        fn delta(f64)
+    }
 }
 
 impl<R: Respond> Default for Rga<R> {
     fn default() -> Self {
         Self {
-            base: BasicSetting {
-                pop_num: 500,
-                ..Default::default()
-            },
             cross: 0.95,
             mutate: 0.05,
             win: 0.95,
@@ -45,15 +46,7 @@ impl<R: Respond> Default for Rga<R> {
 impl<R: Respond> Setting for Rga<R> {
     type Algorithm = Method<R>;
 
-    fn base(&self) -> &BasicSetting {
-        &self.base
-    }
-
-    fn base_mut(&mut self) -> &mut BasicSetting {
-        &mut self.base
-    }
-
-    fn create(self) -> Self::Algorithm {
+    fn algorithm(self) -> Self::Algorithm {
         Method {
             cross: self.cross,
             mutate: self.mutate,
@@ -63,14 +56,12 @@ impl<R: Respond> Setting for Rga<R> {
             pool_new: Array2::zeros((1, 1)),
         }
     }
-}
 
-#[inline(always)]
-fn check<F: ObjFunc>(ctx: &Context<F>, s: usize, v: f64) -> f64 {
-    if ctx.ub(s) < v || v < ctx.lb(s) {
-        rand_float(ctx.lb(s), ctx.ub(s))
-    } else {
-        v
+    fn default_basic() -> BasicSetting {
+        BasicSetting {
+            pop_num: 500,
+            ..Default::default()
+        }
     }
 }
 
@@ -90,7 +81,7 @@ impl<R: Respond> Method<R> {
         F: ObjFunc<Respond = R>,
     {
         for i in (0..(ctx.pop_num() - 1)).step_by(2) {
-            if !maybe(self.cross) {
+            if !ctx.rng.maybe(self.cross) {
                 continue;
             }
             use TmpId::*;
@@ -107,12 +98,16 @@ impl<R: Respond> Method<R> {
                 .map(|id| {
                     let mut v = Array1::zeros(ctx.dim());
                     for s in 0..ctx.dim() {
-                        let variable = match id {
+                        let var = match id {
                             I1 => 0.5 * ctx.pool[[i, s]] + 0.5 * ctx.pool[[i + 1, s]],
                             I2 => 1.5 * ctx.pool[[i, s]] - 0.5 * ctx.pool[[i + 1, s]],
                             I3 => -0.5 * ctx.pool[[i, s]] + 1.5 * ctx.pool[[i + 1, s]],
                         };
-                        v[s] = check(ctx, s, variable);
+                        v[s] = if ctx.ub(s) < var || var < ctx.lb(s) {
+                            ctx.rng.float(ctx.lb(s)..ctx.ub(s))
+                        } else {
+                            var
+                        };
                     }
                     let f = ctx.func.fitness(v.as_slice().unwrap(), &ctx.report);
                     (f, v)
@@ -132,12 +127,11 @@ impl<R: Respond> Method<R> {
             Task::MaxGen(v) if v > 0 => ctx.report.gen as f64 / v as f64,
             _ => 1.,
         };
-        #[cfg(feature = "std")]
-        #[allow(unused)]
+        #[cfg(all(feature = "std", not(feature = "libm")))]
         let pow_f = (1. - r).powf(self.delta);
         #[cfg(feature = "libm")]
         let pow_f = libm::pow(1. - r, self.delta);
-        y * rand() * pow_f
+        y * ctx.rng.rand() * pow_f
     }
 
     fn mutate<F>(&mut self, ctx: &mut Context<F>)
@@ -145,11 +139,11 @@ impl<R: Respond> Method<R> {
         F: ObjFunc<Respond = R>,
     {
         for i in 0..ctx.pop_num() {
-            if !maybe(self.mutate) {
+            if !ctx.rng.maybe(self.mutate) {
                 continue;
             }
-            let s = rand_int(0, ctx.dim());
-            if maybe(0.5) {
+            let s = ctx.rng.int(0..ctx.dim());
+            if ctx.rng.maybe(0.5) {
                 ctx.pool[[i, s]] += self.get_delta(ctx, ctx.ub(s) - ctx.pool[[i, s]]);
             } else {
                 ctx.pool[[i, s]] -= self.get_delta(ctx, ctx.pool[[i, s]] - ctx.lb(s));
@@ -166,10 +160,10 @@ impl<R: Respond> Method<R> {
         for i in 0..ctx.pop_num() {
             let (j, k) = {
                 let mut v = [i, 0, 0];
-                rand_vector(&mut v, 1, 0, ctx.pop_num());
+                ctx.rng.vector(&mut v, 1, 0..ctx.pop_num());
                 (v[1], v[2])
             };
-            if ctx.fitness[j].value() > ctx.fitness[k].value() && maybe(self.win) {
+            if ctx.fitness[j].value() > ctx.fitness[k].value() && ctx.rng.maybe(self.win) {
                 self.fitness_new[i] = ctx.fitness[k].clone();
                 self.pool_new
                     .slice_mut(s![i, ..])
@@ -182,7 +176,8 @@ impl<R: Respond> Method<R> {
             }
             ctx.fitness = self.fitness_new.clone();
             ctx.pool.assign(&self.pool_new);
-            ctx.assign_from_best(rand_int(0, ctx.pop_num()));
+            let i = ctx.rng.int(0..ctx.pop_num());
+            ctx.assign_from_best(i);
         }
     }
 }
